from cyxtrax.common.mesh_object import MeshObject
from pathlib import Path
import numpy as np
import h5py
import json


def to_dict(mesh_object=MeshObject) -> dict:
    return mesh_object.as_dict()


def save_atlas(
    mesh_list: list[MeshObject],
    maps: np.ndarray,
    map_position: np.ndarray,
    atlas_name: str,
    save_folder: Path,
):
    save_path = save_folder / f"{atlas_name}.h5"
    file = h5py.File(save_path, "w")

    x_px = maps.shape[0]
    y_px = maps.shape[1]
    number_of_maps = maps.shape[2]

    projection = file.require_dataset(
        "maps",
        shape=(
            x_px,
            y_px,
            number_of_maps,
        ),  # Initial shape with third dimension as 0
        maxshape=(
            x_px,
            y_px,
            number_of_maps,
        ),
        dtype=np.float32,
    )

    projection_points = file.require_dataset(
        "positions",
        shape=(3, number_of_maps),  # Initial shape with third dimension as 0
        maxshape=(3, number_of_maps),
        dtype=np.float32,
    )
    if isinstance(mesh_list, dict):
        mesh_list_dict = mesh_list
    else:
        mesh_list_dict = list(map(to_dict, mesh_list))
    file.attrs["mesh_list"] = json.dumps(mesh_list_dict)

    projection[:] = maps
    projection_points[:] = map_position
